/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.glassfish.scripting.jruby.monitor;

import com.sun.grizzly.tcp.http11.GrizzlyRequest;
import com.sun.grizzly.tcp.http11.GrizzlyResponse;
import org.glassfish.api.statistics.CountStatistic;
import org.glassfish.api.statistics.impl.CountStatisticImpl;
import org.glassfish.gmbal.Description;
import org.glassfish.gmbal.ManagedAttribute;
import org.glassfish.gmbal.ManagedObject;
import org.glassfish.probe.provider.annotations.ProbeListener;
import org.glassfish.probe.provider.annotations.ProbeParam;
import org.glassfish.scripting.monitor.utils.TimeStat;

import java.util.HashMap;
import java.util.logging.Logger;


/**
 * The stats provider per application. This means It should be created by JRubyApplication during deployment.
 *
 * @author Vivek Pandey
 */
@ManagedObject
@Description("Ruby application HTTP stats")
public class JRubyHttpStatsProvider {

    private final Logger logger = Logger.getLogger(JRubyHttpStatsProvider.class.getName());

    private final CountStatisticImpl requestCount = new CountStatisticImpl("requestCounter", CountStatisticImpl.UNIT_COUNT, "Number of HTTP requests received");
    private final CountStatisticImpl errorCount = new CountStatisticImpl("ErrorCount", "count", "Number of responses with a status code greater than 400");
    private final CountStatisticImpl error200Count = new CountStatisticImpl("Count200", "count", "Number of responses with a status code equal to 200");
    private final CountStatisticImpl error2xxCount = new CountStatisticImpl("Count2xx", "count", "Number of responses with a status code in the 2xx range");
    private final CountStatisticImpl error302Count = new CountStatisticImpl("Count302", "count", "Number of responses with a status code equal to 302");
    private final CountStatisticImpl error304Count = new CountStatisticImpl("Count304", "count", "Number of responses with a status code equal to 304");
    private final CountStatisticImpl error3xxCount = new CountStatisticImpl("Count3xx", "count", "Number of responses with a status code in the 3xx range");
    private final CountStatisticImpl error400Count = new CountStatisticImpl("Count400", "count", "Number of responses with a status code equal to 400");
    private final CountStatisticImpl error401Count = new CountStatisticImpl("Count401", "count", "Number of responses with a status code equal to 401");
    private final CountStatisticImpl error403Count = new CountStatisticImpl("Count403", "count", "Number of responses with a status code equal to 403");
    private final CountStatisticImpl error404Count = new CountStatisticImpl("Count404", "count", "Number of responses with a status code equal to 404");
    private final CountStatisticImpl error4xxCount = new CountStatisticImpl("Count4xx", "count", "Number of responses with a status code in the 4xx range");
    private final CountStatisticImpl error503Count = new CountStatisticImpl("Count503", "count", "Number of responses with a status code equal to 503");
    private final CountStatisticImpl error5xxCount = new CountStatisticImpl("Count5xx", "count", "Number of responses with a status code in the 5xx range");
    private final CountStatisticImpl errorOtherCount = new CountStatisticImpl("CountOther", "count", "Number of responses with other status codes");


    private final HashMap<String, CountStatisticImpl> errorMap = new HashMap<String, CountStatisticImpl>();
    private TimeStat timer;


    private final String appName;
    private final String contextPath;
    private String applicationAddress;

    public JRubyHttpStatsProvider(String appName, String contextPath) {
        this.appName = appName;
        this.contextPath = contextPath;
        errorMap.put("200", error200Count);
        errorMap.put("2", error2xxCount);
        errorMap.put("302", error302Count);
        errorMap.put("304", error304Count);
        errorMap.put("3", error3xxCount);
        errorMap.put("400", error400Count);
        errorMap.put("401", error401Count);
        errorMap.put("403", error403Count);
        errorMap.put("404", error404Count);
        errorMap.put("4", error4xxCount);
        errorMap.put("503", error503Count);
        errorMap.put("5", error5xxCount);
        errorMap.put("9", errorOtherCount);
    }


    @ProbeListener("glassfish:jruby:http-stats:requestStartEvent")
    public void requestStartEvent(
            @ProbeParam("contextPath") String contextPath,
            @ProbeParam("request") GrizzlyRequest request,
            @ProbeParam("response") GrizzlyResponse response
    ) {
        if (timer == null) {
            timer = new TimeStat();
        }
        if (applicationAddress == null) {
            this.applicationAddress = request.getLocalAddr();
        }
        this.requestCount.increment();
    }

    @ProbeListener("glassfish:jruby:http-stats:requestEndEvent")
    public void requestEndEvent(
            @ProbeParam("contextPath") String contextPath,
            @ProbeParam("request") GrizzlyRequest request,
            @ProbeParam("response") GrizzlyResponse response
    ) {
        this.requestCount.decrement();
        incrementStatsCounter(response.getStatus());
    }

    @ManagedAttribute
    @Description("Total requests processed so far")
    public String getAddress() {
        return applicationAddress;
    }

    @ManagedAttribute
    @Description("Total requests processed so far")
    public CountStatistic getRequestCount() {
        return requestCount;
    }

    @ManagedAttribute
    @Description("Requests/Second")
    public int getRequestPerSecond() {
        if (timer == null)
            return 0;
        return (int) (requestCount.getCount() / timer.getTotalTime(TimeStat.UNIT.SECOND));
    }

    @ManagedAttribute
    @Description("Average request processing time in milliseconds")
    public long getAverageProcessingTime() {
        if (timer == null || requestCount.getCount() == 0)
            return 0;
        return timer.getTotalTime(TimeStat.UNIT.MILLISECOND) / requestCount.getCount();
    }


    @ManagedAttribute(id = "errorcount")
    @Description("")
    public CountStatistic getErrorCount() {
        return errorCount;
    }

    @ManagedAttribute(id = "count200")
    @Description("")
    public CountStatistic getError200Count() {
        return error200Count;
    }

    @ManagedAttribute(id = "count2xx")
    @Description("")
    public CountStatistic getError2xxCount() {
        return error2xxCount;
    }

    @ManagedAttribute(id = "count302")
    @Description("")
    public CountStatistic getError302Count() {
        return error302Count;
    }

    @ManagedAttribute(id = "count304")
    @Description("")
    public CountStatistic getError304Count() {
        return error304Count;
    }

    @ManagedAttribute(id = "count3xx")
    @Description("")
    public CountStatistic getError3xxCount() {
        return error3xxCount;
    }

    @ManagedAttribute(id = "count400")
    @Description("")
    public CountStatistic getError400Count() {
        return error400Count;
    }

    @ManagedAttribute(id = "count401")
    @Description("")
    public CountStatistic getError401Count() {
        return error401Count;
    }

    @ManagedAttribute(id = "count403")
    @Description("")
    public CountStatistic getError403Count() {
        return error403Count;
    }

    @ManagedAttribute(id = "count404")
    @Description("")
    public CountStatistic getError404Count() {
        return error404Count;
    }

    @ManagedAttribute(id = "count4xx")
    @Description("")
    public CountStatistic getError4xxCount() {
        return error4xxCount;
    }

    @ManagedAttribute(id = "count503")
    @Description("")
    public CountStatistic getError503Count() {
        return error503Count;
    }

    @ManagedAttribute(id = "count5xx")
    @Description("")
    public CountStatistic getError5xxCount() {
        return this.error5xxCount;
    }

    @ManagedAttribute(id = "countother")
    @Description("")
    public CountStatistic getErrorOtherCount() {
        return this.errorOtherCount;
    }

    private void incrementStatsCounter(int statusCode) {
        CountStatisticImpl errorCounter = errorMap.get(Integer.toString(statusCode));
        if (errorCounter == null)
            errorCounter = errorMap.get(Integer.toString(Math.round(statusCode / 100)));
        if (errorCounter == null)
            errorCounter = errorMap.get("9");
        errorCounter.increment();
        if (statusCode > 400)
            errorCount.increment();
    }

}
